@@ -15,7 +15,10 @@ from requests.exceptions import RequestException
 
 from .credentials import Credentials, guess_credentials
 
+log_handler = logging.StreamHandler()
+log_handler.setLevel(logging.DEBUG)
 logger = logging.getLogger('las')
+logger.addHandler(log_handler)
 Content = Union[bytes, bytearray, str, Path, io.IOBase]
 Queryparam = Union[str, List[str]]
 
@@ -153,7 +156,7 @@ class Client:
         return _json_decode(response)
 
     def create_asset(self, content: Content, **optional_args) -> Dict:
-        """Creates an asset handle, calls the POST /assets endpoint.
+        """Creates an asset, calls the POST /assets endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
@@ -162,9 +165,9 @@ class Client:
         :param content: Content to POST
         :type content: Content
         :param name: Name of the asset
-        :type name: str
+        :type name: Optional[str]
         :param description: Description of the asset
-        :type description: str
+        :type description: Optional[str]
         :return: Asset response from REST API
         :rtype: dict
 
@@ -185,9 +188,9 @@ class Client:
         >>> client.list_assets()
 
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Assets response from REST API without the content of each asset
         :rtype: dict
 
@@ -201,7 +204,7 @@ class Client:
         return self._make_request(requests.get, '/assets', params=params)
 
     def get_asset(self, asset_id: str) -> Dict:
-        """Get asset from the REST API, calls the GET /assets/{assetId} endpoint.
+        """Get asset, calls the GET /assets/{assetId} endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
@@ -218,7 +221,7 @@ class Client:
         return self._make_request(requests.get, f'/assets/{asset_id}')
 
     def update_asset(self, asset_id: str, **optional_args) -> Dict:
-        """Updates an asset, calls the PATCH /assets/assetId endpoint.
+        """Updates an asset, calls the PATCH /assets/{assetId} endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
@@ -227,11 +230,11 @@ class Client:
         :param asset_id: Id of the asset
         :type asset_id: str
         :param content: Content to PATCH
-        :type content: Content
+        :type content: Optional[Content]
         :param name: Name of the asset
-        :type name: str
+        :type name: Optional[str]
         :param description: Description of the asset
-        :type description: str
+        :type description: Optional[str]
         :return: Asset response from REST API
         :rtype: dict
 
@@ -252,9 +255,9 @@ class Client:
         >>> client.create_batch(description='<description>')
 
         :param name: Name of the batch
-        :type name: str
+        :type name: Optional[str]
         :param description: Description of the batch
-        :type description: str
+        :type description: Optional[str]
         :return: Batch response from REST API
         :rtype: dict
 
@@ -272,22 +275,22 @@ class Client:
             batch_id: str = None,
             ground_truth: Sequence[Dict[str, str]] = None,
     ) -> Dict:
-        """Creates a document handle, calls the POST /documents endpoint.
+        """Creates a document, calls the POST /documents endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
-        >>> client.create_document(b'<bytes data>', 'image/jpeg', '<consent id>')
+        >>> client.create_document(b'<bytes data>', 'image/jpeg', consent_id='<consent id>')
 
         :param content: Content to POST
         :type content: Content
-        :param content_type: MIME type for the document handle
+        :param content_type: MIME type for the document
         :type content_type: str
-        :param consent_id: Id of the consent that marks the owner of the document handle
-        :type consent_id: str
+        :param consent_id: Id of the consent that marks the owner of the document
+        :type consent_id: Optional[str]
         :param batch_id: Id of the associated batch
-        :type batch_id: str
+        :type batch_id: Optional[str]
         :param ground_truth: List of items {label: value} representing the ground truth values for the document
-        :type ground_truth: Sequence[Dict[str, str]]
+        :type ground_truth: Optional[Sequence[Dict[str, str]]]
         :return: Document response from REST API
         :rtype: dict
 
@@ -318,13 +321,13 @@ class Client:
         >>> client.list_documents(batch_id='<batch_id>', consent_id='<consent_id>')
 
         :param batch_id: Ids of batches that contains the documents of interest
-        :type batch_id: Queryparam
-        :param consent_id: Ids of the consents that marks the owner of the document handle
-        :type consent_id: Queryparam
+        :type batch_id: Optional[Queryparam]
+        :param consent_id: Ids of the consents that marks the owner of the document
+        :type consent_id: Optional[Queryparam]
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Documents response from REST API
         :rtype: dict
 
@@ -344,10 +347,10 @@ class Client:
 
         >>> from las.client import Client
         >>> client = Client()
-        >>> client.delete_documents('<consent id>')
+        >>> client.delete_documents(consent_id='<consent id>')
 
-        :param consent_id: Ids of the consents that marks the owner of the document handle
-        :type consent_id: Queryparam
+        :param consent_id: Ids of the consents that marks the owner of the document
+        :type consent_id: Optional[Queryparam]
         :return: Documents response from REST API
         :rtype: dict
 
@@ -357,11 +360,11 @@ class Client:
         return self._make_request(requests.delete, '/documents', params={'consentId': consent_id})
 
     def get_document(self, document_id: str) -> Dict:
-        """Get document from the REST API, calls the GET /documents/{documentId} endpoint.
+        """Get document, calls the GET /documents/{documentId} endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
-        >>> client.get_document(document_id='<document id>')
+        >>> client.get_document('<document id>')
 
         :param document_id: Id of the document
         :type document_id: str
@@ -374,8 +377,8 @@ class Client:
         return self._make_request(requests.get, f'/documents/{document_id}')
 
     def update_document(self, document_id: str, ground_truth: Sequence[Dict[str, Union[Optional[str], bool]]]) -> Dict:
-        """Post ground truth to the REST API, calls the PATCH /documents/{documentId} endpoint.
-        Posting ground truth means posting the ground truth data for the particular document.
+        """Update ground truth for a document, calls the PATCH /documents/{documentId} endpoint.
+        Updating ground truth means adding the ground truth data for the particular document.
         This enables the API to learn from past mistakes.
 
         >>> from las.client import Client
@@ -395,6 +398,23 @@ class Client:
         """
         return self._make_request(requests.patch, f'/documents/{document_id}', body={'groundTruth': ground_truth})
 
+    def get_log(self, log_id) -> Dict:
+        """get log, calls the GET /logs/{logId} endpoint.
+
+        >>> from las.client import Client
+        >>> client = Client()
+        >>> client.get_log('<log_id>')
+
+        :param log_id: Id of the log
+        :type log_id: str
+        :return: Log response from REST API
+        :rtype: dict
+
+        :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
+ :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
+        """
+        return self._make_request(requests.get, f'/logs/{log_id}')
+
     def list_models(self, *, max_results: Optional[int] = None, next_token: Optional[str] = None) -> Dict:
         """List models available, calls the GET /models endpoint.
 
@@ -403,9 +423,9 @@ class Client:
         >>> client.list_models()
 
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Models response from REST API without the content of each model
         :rtype: dict
 
@@ -437,10 +457,10 @@ class Client:
         :param model_id: Id of the model to use for inference
         :type model_id: str
         :param max_pages: Maximum number of pages to run predictions on
-        :type max_pages: int
+        :type max_pages: Optional[int]
         :param auto_rotate: Whether or not to let the API try different rotations on\
  the document when running predictions
-        :type auto_rotate: bool
+        :type auto_rotate: Optional[bool]
         :return: Prediction response from REST API
         :rtype: dict
 
@@ -463,9 +483,9 @@ class Client:
         >>> client.list_predictions()
 
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Predictions response from REST API without the content of each prediction
         :rtype: dict
 
@@ -479,7 +499,7 @@ class Client:
         return self._make_request(requests.get, '/predictions', params=params)
 
     def create_secret(self, data: dict, **optional_args) -> Dict:
-        """Creates an secret handle, calls the POST /secrets endpoint.
+        """Creates an secret, calls the POST /secrets endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
@@ -489,9 +509,9 @@ class Client:
         :param data: Dict containing the data you want to keep secret
         :type data: str
         :param name: Name of the secret
-        :type name: str
+        :type name: Optional[str]
         :param description: Description of the secret
-        :type description: str
+        :type description: Optional[str]
         :return: Secret response from REST API
         :rtype: dict
 
@@ -512,9 +532,9 @@ class Client:
         >>> client.list_secrets()
 
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Secrets response from REST API without the username of each secret
         :rtype: dict
 
@@ -538,7 +558,7 @@ class Client:
         :param secret_id: Id of the secret
         :type secret_id: str
         :param data: Dict containing the data you want to keep secret
-        :type data: str
+        :type data: Optional[dict]
         :param name: Name of the secret
         :type name: Optional[str]
         :param description: Description of the secret
@@ -562,7 +582,7 @@ class Client:
         parameters: Optional[dict] = None,
         **optional_args,
     ) -> Dict:
-        """Creates a transition handle, calls the POST /transitions endpoint.
+        """Creates a transition, calls the POST /transitions endpoint.
 
         >>> import json
         >>> from pathlib import Path
@@ -570,27 +590,27 @@ class Client:
         >>> client = Client()
         >>> in_schema = {'$schema': 'https://json-schema.org/draft-04/schema#', 'title': 'in', 'properties': {...} }
         >>> out_schema = {'$schema': 'https://json-schema.org/draft-04/schema#', 'title': 'out', 'properties': {...} }
-        >>> # A typical docker transitions
+        >>> # A typical docker transition
         >>> docker_params = {
         >>>     'imageUrl': '<image_url>',
         >>>     'credentials': {'username': '<username>', 'password': '<password>'}
         >>> }
         >>> client.create_transition('docker', in_schema=in_schema, out_schema=out_schema, params=docker_params)
-        >>> # A typical manual transitions
+        >>> # A manual transition with UI
         >>> assets = {'jsRemoteComponent': 'las:asset:<hex-uuid>', '<other asset name>': 'las:asset:<hex-uuid>'}
         >>> manual_params = {'assets': assets}
         >>> client.create_transition('manual', in_schema=in_schema, out_schema=out_schema, params=manual_params)
 
-        :param in_schema: Json-schema that defines the input to the transition
-        :type in_schema: dict
-        :param out_schema: Json-schema that defines the output of the transition
-        :type out_schema: dict
-        :param name: Name of the transition
-        :type name: str
         :param transition_type: Type of transition "docker"|"manual"
         :type transition_type: str
-        :param params: Parameters to the corresponding transition type
-        :type params: Optional[dict]
+        :param in_schema: Json-schema that defines the input to the transition
+        :type in_schema: Optional[dict]
+        :param out_schema: Json-schema that defines the output of the transition
+        :type out_schema: Optional[dict]
+        :param name: Name of the transition
+        :type name: Optional[str]
+        :param parameters: Parameters to the corresponding transition type
+        :type parameters: Optional[dict]
         :param description: Description of the transition
         :type description: Optional[str]
         :return: Transition response from REST API
@@ -622,11 +642,11 @@ class Client:
         >>> client.list_transitions('<transition_type>')
 
         :param transition_type: Types of transitions
-        :type transition_type: Queryparam
+        :type transition_type: Optional[Queryparam]
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Transitions response from REST API
         :rtype: dict
 
@@ -641,6 +661,23 @@ class Client:
         }
         return self._make_request(requests.get, url, params=dictstrip(params))
 
+    def get_transition(self, transition_id: str) -> Dict:
+        """Get the transition with the provided transition_id, calls the GET /transitions/{transitionId} endpoint.
+
+        >>> from las.client import Client
+        >>> client = Client()
+        >>> client.get_transition('<transition_id>')
+
+        :param transition_id: Id of the transition
+        :type transition_id: str
+        :return: Transition response from REST API
+        :rtype: dict
+
+        :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
+ :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
+        """
+        return self._make_request(requests.get, f'/transitions/{transition_id}')
+
     def update_transition(
         self,
         transition_id: str,
@@ -649,7 +686,7 @@ class Client:
         out_schema: Optional[dict] = None,
         **optional_args,
     ) -> Dict:
-        """Creates a transition handle, calls the PATCH /transitions/{transitionId} endpoint.
+        """Updates a transition, calls the PATCH /transitions/{transitionId} endpoint.
 
         >>> import json
         >>> from pathlib import Path
@@ -698,6 +735,24 @@ class Client:
         endpoint = f'/transitions/{transition_id}/executions'
         return self._make_request(requests.post, endpoint, body={})
 
+    def delete_transition(self, transition_id: str) -> Dict:
+        """Delete the transition with the provided transition_id, calls the DELETE /transitions/{transitionId} endpoint.
+           Will fail if transition is in use by one or more workflows.
+
+        >>> from las.client import Client
+        >>> client = Client()
+        >>> client.delete_transition('<transition_id>')
+
+        :param transition_id: Id of the transition
+        :type transition_id: str
+        :return: Transition response from REST API
+        :rtype: dict
+
+        :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
+ :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
+        """
+        return self._make_request(requests.delete, f'/transitions/{transition_id}')
+
     def list_transition_executions(
         self,
         transition_id: str,
@@ -718,17 +773,17 @@ class Client:
         :param transition_id: Id of the transition
         :type transition_id: str
         :param status: Statuses of the executions
-        :type status: Queryparam
+        :type status: Optional[Queryparam]
         :param order: Order of the executions, either 'ascending' or 'descending'
-        :type order: Optional str
+        :type order: Optional[str]
         :param sort_by: the sorting variable of the executions, either 'endTime', or 'startTime'
-        :type sort_by: Optional str
+        :type sort_by: Optional[str]
         :param execution_id: Ids of the executions
-        :type execution_id: Queryparam
+        :type execution_id: Optional[Queryparam]
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Transition executions responses from REST API
         :rtype: dict
 
@@ -746,6 +801,26 @@ class Client:
         }
         return self._make_request(requests.get, url, params=dictstrip(params))
 
+    def get_transition_execution(self, transition_id: str, execution_id: str) -> Dict:
+        """Get an execution of a transition, calls the GET /transitions/{transitionId}/executions/{executionId} endpoint
+
+        >>> from las.client import Client
+        >>> client = Client()
+        >>> client.get_transition_execution('<transition_id>', '<execution_id>')
+
+        :param transition_id: Id of the transition
+        :type transition_id: str
+        :param execution_id: Id of the executions
+        :type execution_id: str
+        :return: Transition execution responses from REST API
+        :rtype: dict
+
+        :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
+ :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
+        """
+        url = f'/transitions/{transition_id}/executions/{execution_id}'
+        return self._make_request(requests.get, url)
+
     def update_transition_execution(
         self,
         transition_id: str,
@@ -772,9 +847,9 @@ class Client:
         :param status: Status of the execution 'succeeded|failed'
         :type status: str
         :param output: Output from the execution, required when status is 'succeded'
-        :type output: str
+        :type output: Optional[str]
         :param error: Error from the execution, required when status is 'failed', needs to contain 'message'
-        :type error: str
+        :type error: Optional[str]
         :return: Transition execution response from REST API
         :rtype: dict
 
@@ -789,22 +864,30 @@ class Client:
         }
         return self._make_request(requests.patch, url, body=dictstrip(body))
 
-    def create_user(self, email: str) -> Dict:
+    def create_user(self, email: str, **optional_args) -> Dict:
         """Creates a new user, calls the POST /users endpoint.
 
         >>> from las.client import Client
         >>> client = Client()
-        >>> client.create_user('<email>')
+        >>> client.create_user('<email>', name='John Doe')
 
         :param email: Email to the new user
         :type email: str
+        :param name: Name of the user
+        :type name: Optional[str]
+        :param avatar: base64 encoded JPEG avatar of the user
+        :type avatar: Optional[str]
         :return: User response from REST API
         :rtype: dict
 
         :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
  :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
         """
-        return self._make_request(requests.post, '/users', body={'email': email})
+        body = {
+            'email': email,
+            **optional_args,
+        }
+        return self._make_request(requests.post, '/users', body=body)
 
     def list_users(self, *, max_results: Optional[int] = None, next_token: Optional[str] = None) -> Dict:
         """List users, calls the GET /users endpoint.
@@ -814,9 +897,9 @@ class Client:
         >>> client.list_users()
 
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Users response from REST API
         :rtype: dict
 
@@ -846,6 +929,28 @@ class Client:
         """
         return self._make_request(requests.get, f'/users/{user_id}')
 
+    def update_user(self, user_id: str, **optional_args) -> Dict:
+        """Updates a user, calls the PATCH /users/{userId} endpoint.
+
+        >>> from las.client import Client
+        >>> client = Client()
+        >>> client.update_user('<user id>', name='John Doe')
+
+        :param user_id: Id of the user
+        :type user_id: str
+        :param name: Name of the user
+        :type name: Optional[str]
+        :param avatar: base64 encoded JPEG avatar of the user
+        :type avatar: Optional[str]
+        :return: User response from REST API
+        :rtype: dict
+
+        :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
+ :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
+        """
+
+        return self._make_request(requests.patch, f'/users/{user_id}', body=optional_args)
+
     def delete_user(self, user_id: str) -> Dict:
         """Delete the user with the provided user_id, calls the DELETE /users/{userId} endpoint.
 
@@ -865,6 +970,7 @@ class Client:
 
     def create_workflow(self, specification: dict, *, error_config: Optional[dict] = None, **optional_args) -> Dict:
         """Creates a new workflow, calls the POST /workflows endpoint.
+        Check out Lucidtech's tutorials for more info on how to create a workflow.
 
         >>> from las.client import Client
         >>> from pathlib import Path
@@ -877,11 +983,11 @@ class Client:
             currently supporting ASL: https://states-language.net/spec.html
         :type specification: dict
         :param name: Name of the workflow
-        :type name: str
+        :type name: Optional[str]
         :param description: Description of the workflow
-        :type description: str
+        :type description: Optional[str]
         :param error_config: Configuration of error handler
-        :type error_config: dict
+        :type error_config: Optional[dict]
         :return: Workflow response from REST API
         :rtype: dict
 
@@ -904,9 +1010,9 @@ class Client:
         >>> client.list_workflows()
 
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Workflows response from REST API
         :rtype: dict
 
@@ -919,8 +1025,25 @@ class Client:
         }
         return self._make_request(requests.get, '/workflows', params=params)
 
+    def get_workflow(self, workflow_id: str) -> Dict:
+        """Get the workflow with the provided workflow_id, calls the GET /workflows/{workflowId} endpoint.
+
+        >>> from las.client import Client
+        >>> client = Client()
+        >>> client.get_workflow('<workflow_id>')
+
+        :param workflow_id: Id of the workflow
+        :type workflow_id: str
+        :return: Workflow response from REST API
+        :rtype: dict
+
+        :raises: :py:class:`~las.InvalidCredentialsException`, :py:class:`~las.TooManyRequestsException`,\
+ :py:class:`~las.LimitExceededException`, :py:class:`requests.exception.RequestException`
+        """
+        return self._make_request(requests.get, f'/workflows/{workflow_id}')
+
     def update_workflow(self, workflow_id: str, **optional_args) -> Dict:
-        """Creates a workflow handle, calls the PATCH /workflows/{workflowId} endpoint.
+        """Updates a workflow, calls the PATCH /workflows/{workflowId} endpoint.
 
         >>> import json
         >>> from pathlib import Path
@@ -1000,15 +1123,15 @@ class Client:
         :param workflow_id: Id of the workflow
         :type workflow_id: str
         :param order: Order of the executions, either 'ascending' or 'descending'
-        :type order: Optional str
+        :type order: Optional[str]
         :param sort_by: the sorting variable of the executions, either 'endTime', or 'startTime'
-        :type sort_by: Optional str
+        :type sort_by: Optional[str]
         :param status: Statuses of the executions
-        :type status: Queryparam
+        :type status: Optional[Queryparam]
         :param max_results: Maximum number of results to be returned
-        :type max_results: int
+        :type max_results: Optional[int]
         :param next_token: A unique token for each page, use the returned token to retrieve the next page.
-        :type next_token: str
+        :type next_token: Optional[str]
         :return: Workflow executions responses from REST API
         :rtype: dict
 
